import 'dart:io';

import 'package:jinja/jinja.dart';
import 'package:shelf/shelf.dart';
import 'package:shelf/shelf_io.dart';

Environment env;

Response renderTemplate(String name, [Map<String, dynamic> data]) =>
    Response.ok(env.getTemplate(name).render(data),
        headers: {'Content-Type': ContentType.html.toString()});

Future<void> main(List<String> args) async {
  env = Environment(loader: FileSystemLoader());
  final handler = Pipeline()
      .addMiddleware(logRequests())
      .addHandler(Cascade().add(index).add(hello).handler);

  final server = await serve(handler, 'localhost', 3000);
  print('Serving at http://${server.address.host}:${server.port}');
}

Response index(Request request) {
  if (request.url.path.isNotEmpty) return Response.notFound(null);
  return renderTemplate('index.html');
}

Response hello(Request request) {
  if (!request.url.path.startsWith('hello')) return Response.notFound(null);
  if (request.url.pathSegments.length == 1)
    return renderTemplate('hello.html', <String, dynamic>{'name': 'user'});
  return renderTemplate(
      'hello.html', <String, dynamic>{'name': request.url.pathSegments[1]});
}
