import 'runtime.dart';
import 'utils.dart';

num doAbs(num value) => value.abs();

dynamic doAttr(dynamic value, String attr) {
  if (value is Map) return value[attr] ?? getAttr(attr, value);
  return getAttr(attr, value);
}

Iterable<List<dynamic>> doBatch(dynamic value, int lineCount,
    [dynamic fillWith]) sync* {
  List<dynamic> tmp = <dynamic>[];
  for (dynamic item in value) {
    if (tmp.length == lineCount) {
      yield tmp.toList();
      tmp.clear();
    }
    tmp.add(item);
  }
  if (tmp.isNotEmpty) {
    if (fillWith != null && tmp.length < lineCount)
      tmp.addAll(List<dynamic>.filled(lineCount - tmp.length, fillWith));
    yield tmp;
  }
}

dynamic doDefault(dynamic value,
    [dynamic defaultValue = '', bool boolean = false]) {
  if (boolean) return asBool(value) ? value : defaultValue;
  print(['"$value"', '"$defaultValue"']);
  return value is! Undefined ? value : defaultValue;
}

String doJoin(Iterable<dynamic> values,
    {String separator = '', String attribute}) {
  if (attribute != null)
    values = values.map<dynamic>((dynamic value) => getAttr(attribute, value));
  return values.join(separator);
}

List<dynamic> doList(dynamic value) {
  if (value is Iterable) return value.toList();
  if (value is String) return value.split('');
  return <dynamic>[value];
}

String doLower(dynamic value) => '$value'.toLowerCase();

dynamic doSubList(dynamic values, int start, int end) {
  if (values is List) return values.sublist(start, end);
  if (values is String) return values.substring(start, end);
  throw ArgumentError.value(values, 'value');
}

dynamic doSum(Iterable<num> values, {String attribute, num start = 0}) {
  if (attribute != null)
    values =
        values.map<num>((dynamic value) => getAttr(attribute, value) as num);
  return values.fold<num>(start, (num prev, num current) => prev + current);
}

const filters = <String, Function>{
  'abs': doAbs,
  'attr': doAttr,
  'batch': doBatch,
  'default': doDefault,
  'e': escape,
  'escape': escape,
  'join': doJoin,
  'list': doList,
  'lower': doLower,
  'sublist': doSubList,
  'sum': doSum,
  // 'capitalize':           do_capitalize,
  // 'center':               do_center,
  // 'count':                len,
  // 'd':                    do_default,
  // 'dictsort':             do_dictsort,
  // 'filesizeformat':       do_filesizeformat,
  // 'first':                do_first,
  // 'float':                do_float,
  // 'forceescape':          do_forceescape,
  // 'format':               do_format,
  // 'groupby':              do_groupby,
  // 'indent':               do_indent,
  // 'int':                  do_int,
  // 'last':                 do_last,
  // 'length':               len,
  // 'title':                do_title,
  // 'map':                  do_map,
  // 'min':                  do_min,
  // 'max':                  do_max,
  // 'pprint':               do_pprint,
  // 'random':               do_random,
  // 'reject':               do_reject,
  // 'rejectattr':           do_rejectattr,
  // 'replace':              do_replace,
  // 'reverse':              do_reverse,
  // 'round':                do_round,
  // 'safe':                 do_mark_safe,
  // 'select':               do_select,
  // 'selectattr':           do_selectattr,
  // 'slice':                do_slice,
  // 'sort':                 do_sort,
  // 'string':               soft_unicode,
  // 'striptags':            do_striptags,
  // 'trim':                 do_trim,
  // 'truncate':             do_truncate,
  // 'unique':               do_unique,
  // 'upper':                do_upper,
  // 'urlencode':            do_urlencode,
  // 'urlize':               do_urlize,
  // 'wordcount':            do_wordcount,
  // 'wordwrap':             do_wordwrap,
  // 'xmlattr':              do_xmlattr,
  // 'tojson':               do_tojson,
};
