import 'runtime.dart';
import 'utils.dart';

dynamic eq(dynamic value, dynamic other) => value == other;
dynamic ge(dynamic value, dynamic other) => value >= other;
dynamic gt(dynamic value, dynamic other) => value > other;
dynamic le(dynamic value, dynamic other) => value <= other;
dynamic lt(dynamic value, dynamic other) => value < other;
dynamic ne(dynamic value, dynamic other) => value != other;

/// Return `true` if the variable is [Function].
bool testCallable(dynamic value) => value is Function;

/// Return `true` if the variable is defined.
///
/// See the `default` filter for a simple way to set undefined
/// variables.
bool testDefined(dynamic value) => asBool(value);

/// Check if a variable is divisible by a number.
bool testDivisibleBy(num value, num divider) {
  if (divider == 0) return false;
  return value % divider == 0;
}

/// Check if the value is escaped.
bool testEscaped(dynamic value) => value is Escaped;

/// Return `true` if the variable is even.
bool testEven(num value) => value % 2 == 0;

/// Check if value is in seq.
bool testIn(dynamic value, dynamic seq) {
  if (seq is Iterable) return seq.contains(value);
  if (seq is Map) return seq.containsKey(value);
  if (seq is String) return seq.contains(value as Pattern);
  return false;
}

/// Check if it's possible to iterate over an object.
bool testIterable(dynamic value) => value is Iterable;

/// Return `true` if the variable is lowercased.
bool testLower(String value) => value == value.toLowerCase();

/// Return `true` if the object is a mapping ([Map] etc.).
bool testMapping(dynamic value) => value is Map;

/// Return `true` if the variable is [Undefined].
bool testNone(dynamic value) => value is Undefined;

/// Return `true` if the variable is a [num].
bool testNumber(dynamic value) => value is num;

/// Return `true` if the variable is odd.
bool testOdd(num value) => value % 2 == 1;

/// Check if an object points to the same memory address than another
/// object.
bool testSameAs(dynamic value, dynamic other) => value == other;

/// Return `true` if the variable is a sequence. Sequences are variables
/// that are [Iterable].
bool testSequence(dynamic seq) {
  if (seq is Iterable || seq is Map || seq is String) return true;
  return false;
}

/// Return `true` if the object is a [String].
bool testString(dynamic value) => value is String;

/// Like `defined` but the other way round.
bool testUndefined(dynamic value) => !asBool(value);

/// Return `true` if the variable is uppercased.
bool testUpper(String value) => value == value.toUpperCase();

const tests = <String, Function>{
  'callable': testCallable,
  'defined': testDefined,
  'divisibleby': testDivisibleBy,
  'eq': eq,
  'equalto': eq,
  'escaped': testEscaped,
  'even': testEven,
  'ge': ge,
  'greaterthan': gt,
  'gt': gt,
  'in': testIn,
  'iterable': testIterable,
  'le': le,
  'lessthan': lt,
  'lower': testLower,
  'lt': lt,
  'mapping': testMapping,
  'ne': ne,
  'none': testNone,
  'number': testNumber,
  'odd': testOdd,
  'sameas': testSameAs,
  'sequence': testSequence,
  'string': testString,
  'undefined': testUndefined,
  'upper': testUpper,
};
